#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _copyB, fact
from bs4 import BeautifulSoup
from telegram_util import matchKey

def _tagReplace(soup, args = {}):
	wrap_with_i = [
		soup.find_all("div"),
		soup.find_all("span"),
		soup.find_all("p"),
	]
	for l in wrap_with_i:
		for item in l:
			attrs = str(item.attrs)
			attrs = attrs.replace(": ", ":")
			if 'font-style:italic' in attrs:
				wrapper = fact().new_tag("i")
				wrapper.append(_copyB(item))
				item.replace_with(wrapper)
	wrap_with_p = [
		soup.find_all("div", class_="article-paragraph"),
		soup.find_all("section"),
		[x for x in soup.children if isinstance(x, str) and x[:3] != 'doc'],
	]
	for l in wrap_with_p:
		for item in l:
			wrapper = fact().new_tag("p")
			wrapper.append(_copyB(item))
			item.replace_with(wrapper)
	for l in soup.find_all("p"):
		children = list(l.children)
		if len(children) != 1 or not isinstance(children[0], str):
			continue
		to_replace = None
		if l.parent.name == 'blockquote': # douban status specific
			to_replace = l.parent
		if matchKey(str(l.parent.attrs), ['review-content', 'note']): # douban reviews.notes specific
			to_replace = l
		if not to_replace or len(list(to_replace.children)) != 1:
			continue
		paragraphs = ''.join(['<p>%s</p>' % x for x in children[0].split('\n') 
			if x.strip()])
		to_replace.replace_with(BeautifulSoup("<p>%s</p>" % paragraphs, features="lxml"))
	if args.get('list_replace'):
		to_remove_tags = [
			soup.find_all("li"),
			soup.find_all("ul")
		]
		for l in to_remove_tags:
			for item in l:
				item.name = p
	return soup


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from telegram_util import matchKey
from .images import _yieldPossibleImg

def _replaceOfftopicLink(soup, args = {}):
	for link in soup.find_all("a"):
		if matchKey(link.text, ['英文版']):
			continue
		img = next(_yieldPossibleImg(link), None)
		if img:
			link.replace_with(img)
			continue
		if link.text and link.text.strip():
			link.replace_with(link.text)
			continue
		link.decompose()
	return soup


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import _seemsValidText
from telegram_util import matchKey

SHORT_ARTICLE = ['每日书摘']

def _seemsValidRawArticle(soup, text_limit = 500):
	if not _seemsValidText(soup, limit = text_limit):
		return False
	return not not soup.find('img')

def _getInnerArticle_(soup, domain):	
	applicators = [
		lambda x: x.find("article"),
		lambda x: x.find("article"),
		lambda x: x.find("div", {"id" : "js_content"}),
		lambda x: x.find("div", class_ = "story-body__inner"),
		lambda x: x.find("div", class_ = "answercell"),
		lambda x: x.find("div", class_ = "post-text"),
		lambda x: x.find("div", {"id" : "bodyContent"}),
		lambda x: x.find("div", {"id" : "content_JS"}),
		lambda x: x.find("div", class_ = "main-post"),
		lambda x: x.find("div", class_ = "article"),
		lambda x: x.find("div", class_ = "field-name-body"),
		lambda x: x.find("div", class_ = "content"),
		lambda x: x.find("div", class_ = "u-content"),
		lambda x: x.find("div", class_ = "RichContent-inner"),
		lambda x: x.find("div", class_ = "entry-content"),
	]
	domain_specific_applicators = {
		'': [lambda x: x.find("body")],
		'matters': [lambda x: x.find("div", class_ = "u-content")],
		'douban': [
			lambda x: x.find("div", {"id" : "link-report"}),
			lambda x: x.find("div", class_ = "note"),
			lambda x: x.find("div", class_ = "review-content"),
			lambda x: x.find("div", class_ = "status-wrapper"),],
		'thepaper': [
			lambda x: x.find("div", class_ = "news_txt"),
			lambda x: x.find('div', class_ = 'news_part')],
		'gravitysworm': [lambda x: x.find("div", class_ = "copy")],
	}
	is_short = matchKey(soup.text, SHORT_ARTICLE)
	text_limit = 150 if is_short else 500
	for applicator in applicators:
		candidate = applicator(soup)
		if _seemsValidRawArticle(candidate, text_limit = text_limit):
			soup = candidate
	for d, applicators in domain_specific_applicators.items():
		if d in domain:
			for applicator in applicators:
				candidate = applicator(soup)
				if candidate:
					soup = candidate
	return soup

def _getInnerArticle(soup, domain):
	all_content = str(soup)
	inner = _getInnerArticle_(soup, domain)
	index = all_content.find(str(inner))
	if index == -1:
		return inner, None
	return inner, all_content[:index]


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from telegram_util import matchKey
from .common import fact, _wrap
from .images import _yieldPossibleImg
from bs4 import Comment
import sys

OFFTOPIC_TAG = ['small', 'address', 'meta', 'script', 'sup']

OFFTOPIC_ATT = [
	'latest', 'more', 'button', 'facebook',
	'cn-carousel-medium-strip', 'video__end-slate__top-wrapper', 'metadata', 
	'el__article--embed', 'signup', 'related', 'disclaimer', 'off-screen', 
	'story-body__unordered-list', 'story-image-copyright', 'article-header', 
	'top-wrapper', 'bottom-of-article', 'bottom-wrapper', 'linkList', 
	'display:none;', 'accordion', 'el-editorial-source', 'video__end-slate__tertiary-title',
	'adblocker', 'tagline', 'navmenu', 'topHeader', 'Post Bottom',
	't_callout', 'add-interest', 'bb-newsletter', 'popover', 'toast', 'after-article', 
	'submeta', 'rich-link__container', 'content__meta-container', 'mw-editsection',
	'noprint', 'jump-to-nav', 'toctitle', 'reflist', 'contentSub',
	'参考文献', '引用', '网页', 'printfooter', 'catlinks', 'footer', 'Post Bottom',
	'ad-unit', 'box-ad', "class : ['tabs']", "class : ['hd']", 'NYT_BELOW_MAIN_CONTENT_REGION',
	'NYT_TOP_BANNER_REGION', 'g-inlineguide', 'entry-meta-bar', 'extra-hatom-entry-title',
	'list-unstyled', 'hide_word', 'single-header', 'post-pagination',
	'weui-dialog', 'post-title', 'post-topic', 'byline-date', 'post-print',
	'mailing-list-popup', 'taxonomies', 'authors'
]

OFFTOPIC_ATT_WITH_EXCEPTION = {
	'sidebar': ['no-sidebar', 'one-sidebar', 'penci-main', 'id : page'],
	'hidden': ['lazy', 'false', 'label-hidden', 'rich_media_content', 'start', 'overflow'],
	'navigation': ['navigation-border-thin-decoration'],
	'copyright': ['and'],
	'navbar': ['single-post']
}

OFFTOPIC_CLASSES = ['ads']

DIV_AD_WORDS = [
	'中文简报',
	'@nytimes'
]

P_AD_WORDS = [
	'The Times is committed', 
	'Follow The New York Times',
	'Love HuffPost',
	'discounted subscription today'
]

def _isOffTopic(attrs):
	if not attrs:
		return 
	r = []
	for k, v in attrs.items():
		if matchKey(k, ['href', 'src', 'url', 'alt', 'data', 'xmlns:fb']):
			continue
		r.append(str(k) + ' : ' + str(v))
	r = '\n'.join(r)
	for att in OFFTOPIC_ATT:
		if att in r:
			return att
	for att in OFFTOPIC_ATT_WITH_EXCEPTION:
		if att in r and not matchKey(r, OFFTOPIC_ATT_WITH_EXCEPTION[att]):
			return att
	return 

def _decompose(item):
	if 'offtopic' in str(sys.argv) and item.text and len(item.text) > 500:
		print('decomposing long text: ', item.attrs, ' '.join(item.text[:100].split()))
		if item.name in OFFTOPIC_TAG:
			print(item.name)
		if _isOffTopic(item.attrs):
			print(_isOffTopic(item.attrs))
	item.decompose()

def _decompseAds(soup):
	for item in soup.find_all("div", class_="article-paragraph"):
		if matchKey(item.text, DIV_AD_WORDS):
			_decompose(item)
	for item in soup.find_all("p"):
		if matchKey(item.text, P_AD_WORDS) or item.text in ['广告']:
			_decompose(item)

def _decomposeOfftopic(soup, url, args = {}):
	for item in list(soup.find_all())[1:]:
		if _isOffTopic(item.attrs) or \
			item.name in OFFTOPIC_TAG:
			_decompose(item)

	for c in OFFTOPIC_CLASSES:
		for item in soup.find_all(class_=c):
			_decompose(item)

	if not matchKey(url, ['medium']):
		for item in soup.find_all('h1'):
			_decompose(item)

	_decompseAds(soup)

	for item in soup.find_all(string=lambda text: isinstance(text, Comment)):
		item.extract()

	for item in soup.find_all("header"):
		s = item.find("p", {"id": "article-summary"})
		img = next(_yieldPossibleImg(item), None)
		if not s or not s.text:
			if img:
				item.replace_with(img)
			else:
				_decompose(item)
			continue
		if not img:
			item.replace_with(s)
			continue
		cap = img.find('figcaption')
		if cap and not cap.text:
			cap.replace_with(_wrap('figcaption', s.text))
			item.replace_with(img)
			continue
		if not cap and img.name in ['div', 'figure']:
			img.append(_wrap('figcaption', s.text))
			item.replace_with(img)
			continue
		item.replace_with(_wrap('p', img, s))
	return soup